import netscape.application.*;
import netscape.util.*;

public class CircleView extends View{
  Vector circles;
  Color circlecolor;
  int dragnumber;
  
  public CircleView(Rect rect){
    super(rect);
    circles = new Vector();
    circlecolor = Color.blue;
    setBuffered(true);
  }
  
  public void drawView(Graphics g){
    g.setColor(circlecolor);
    if(!circles.isEmpty()){
      Circle circle;
      for(int k=0;k<circles.size();k++){
        circle = (Circle) circles.elementAt(k);
        g.fillOval(circle.x-circle.r,circle.y-circle.r,
                   2*circle.r,2*circle.r);
      }
    }
  }
  
  public boolean mouseDown(MouseEvent e){
    if(!circles.isEmpty()){
      Circle circle;
      for(int k = 0;k<circles.size();k++){ 
        circle = (Circle) circles.elementAt(k);
        if(circle.isInside(e.x,e.y)){
          if(e.isMetaKeyDown() || e.clickCount() > 1){
	    // There was a right-click or double-click
	    // in an existing circle.  Remove it.
            circles.removeElementAt(k);
	    // Mark the view as dirty so it gets redrawn.
            setDirty(true);
            return false;
          }
          else{
	    // A single click in a circle.
	    // Set it up so the user can drag the circle.
            dragnumber = k;
            return true;
          }
        }
      }
    }
    
    // We're not inside a circle.  Ignore
    // double-clicks and right-clicks.
    if(e.isMetaKeyDown() || e.clickCount() > 1) return false;
    if(e.clickCount() >1) return false;
    circles.addElement(new Circle(e.x,e.y));
    // Make the new circle the circle eligible to
    // be dragged.
    dragnumber = circles.size()-1;
    // redraw the view
    setDirty(true);
    return true;
  }
  
  public void mouseDragged(MouseEvent e){
    Circle circle = (Circle) circles.elementAt(dragnumber);
    // Move the circle and redraw the view.
    circle.moveTo(e.x,e.y);
    setDirty(true);
  }
}

// a simple class representing a circle
class Circle {  
  public int x,y;
  public int r=10;
  
  public Circle(int x, int y){
    this.x = x;
    this.y = y;
  }
  
  public boolean isInside(int a,int b){
    if(((x-a)*(x-a) + (y-b)*(y-b)) <= r*r)
      return true;
    else return false;
  }

  public void moveTo(int x,int y){
    this.x = x;
    this.y = y;
  }
}
