import netscape.application.*;
import netscape.util.*;

// an Application that uses a DrawingSequence
public class SpinnerApp extends Application{
  SpinnerView spinnerview;
  Button spinnerbutton;
  Spinner buttonspinner;
  
  public void init(){
    Rect rect = mainRootView().bounds();
    // create a View subclass that acts as a DrawingSequenceOwner.
    // the SpinnerView automatically starts the DrawingSequence.
    spinnerview = new SpinnerView(rect.width/5,rect.height/5,
				  3*rect.width/5,3*rect.height/5);
    mainRootView().addSubview(spinnerview);

    // create a button.  Button implements DrawingSequenceOwner.
    spinnerbutton = new Button(10,10,130,30);
    buttonspinner = new Spinner(spinnerbutton);
    // set the button's Image and start the playback
    spinnerbutton.setImage(buttonspinner);
    buttonspinner.setPlaybackMode(DrawingSequence.FORWARD_LOOP);
    buttonspinner.setFrameRate(20);  // 20 milliseconds between frames
    buttonspinner.start();  // start the animation
    spinnerbutton.setBuffered(true);
    spinnerbutton.setTitle("Animated Button");
    
    mainRootView().addSubview(spinnerbutton);
  }
}

class SpinnerView extends View implements DrawingSequenceOwner{
  Spinner spinner;
  
  public SpinnerView(int x,int y,int w,int h){
    super(x,y,w,h);
    setBuffered(true);
    spinner = new Spinner(this);  // create the DrawingSequence
    spinner.setFrameRate(50);  // 50 milliseconds between frames
    spinner.setPlaybackMode(DrawingSequence.FORWARD_LOOP);
    spinner.start();   // start the animation!
  }
  
  public void drawView(Graphics g){  
    // get the total number of frames
    double frameCount = (double) spinner.frameCount();
    // get the current frame number
    double currentFrame = (double) spinner.currentFrameNumber();
    // calculate the DrawingSequence's position in the View based on
    // the total number of frames and the current frame number.
    double phase = 2.0*Math.PI*currentFrame/frameCount;
    double dx = ((double) width())*(1.0 + Math.sin(phase))/10.0;
    double dy = ((double) height())*(1.0 + Math.cos(phase))/10.0;
    Rect rect = new Rect((int) dx,(int) dy,8*width()/10,8*height()/10);
    // draw the DrawingSequence at the correct position
    spinner.drawScaled(g,rect);
  }
  
  public void drawingSequenceCompleted(DrawingSequence ds){} 
  
  public void drawingSequenceFrameChanged(DrawingSequence ds){ 
    draw();  // call draw() every time the frame changes.  
  } 
}
    

class Spinner extends DrawingSequence{
  public Spinner(DrawingSequenceOwner dso){
    super(dso);
    setFrameCount(36);
  }
  
  // implement DrawingSequence's three abstract methods
  public void drawAt(Graphics g,int x,int y){
    drawScaled(g,x,y,width(),height());
  }

  public int width(){ return 20;}
  public int height(){ return 20;} 
  
  // implement the drawScaled() method for convenience.  
  public void drawScaled(Graphics g,int x,int y,int w,int h){
    // draw two circular arcs
    int angle = 5*currentFrameNumber();
    g.setColor(Color.green);
    Rect rect = new Rect(x,y,w,h);
    g.drawOval(rect);
    g.fillArc(rect,angle,90);
    g.fillArc(rect,angle+180,90);
  }
}
