import netscape.application.*;
import netscape.util.*;

// A Codable CircleView
public class CodableCircleView extends View implements Codable{
  private final static String CIRCLES = "circles";
  Vector circles;
  Color circlecolor;
  int dragnumber;

  // Need an empty constructor for Codable
  public CodableCircleView(){
    setBuffered(true);
    circlecolor = Color.blue;
  }
  
  public CodableCircleView(Rect rect){
    super(rect);
    circles = new Vector();
    circlecolor = Color.blue;
    setBuffered(true);
  }

  // draw the circles
  public void drawView(Graphics g){
    g.setColor(circlecolor);
    if(!circles.isEmpty()){
      CodableCircle circle;
      for(int k=0;k<circles.size();k++){
        circle = (CodableCircle) circles.elementAt(k);
        g.fillOval(circle.x-circle.r,circle.y-circle.r,2*circle.r,2*circle.r);
      }
    }
  }
  
  // handle the mouse events
  public boolean mouseDown(MouseEvent e){
    if(!circles.isEmpty()){
      CodableCircle circle;
      for(int k = 0;k<circles.size();k++){ 
        circle = (CodableCircle) circles.elementAt(k);
        if(circle.isInside(e.x,e.y)){
          if(e.isMetaKeyDown() || e.clickCount() > 1){
            circles.removeElementAt(k);
            setDirty(true);
            return false;
          }
          else{
            dragnumber = k;
            return true;
          }
        }
      }
    }

    if(e.isMetaKeyDown() || e.clickCount() > 1) return false;
    circles.addElement(new CodableCircle(e.x,e.y));
    dragnumber = circles.size()-1;
    setDirty(true);
    return true;
  }

  public void mouseDragged(MouseEvent e){
    CodableCircle circle = (CodableCircle) circles.elementAt(dragnumber);
    circle.x = e.x;
    circle.y = e.y;
    setDirty(true);
  }

  // Codable interface methods.

  // Called by the Unarchiver.
  public void decode(Decoder decoder) throws CodingException{
    super.decode(decoder);
    circles = (Vector) decoder.decodeObject(CodableCircleView.CIRCLES);
  }

  // Describe the relevant fields to the ClassInfo object
  public void describeClassInfo(ClassInfo classInfo){
    super.describeClassInfo(classInfo);  
    classInfo.addClass("CodableCircleView",1);
    classInfo.addField(CodableCircleView.CIRCLES,Codable.OBJECT_TYPE);
  }

  // Called by the Archiver
  public void encode(Encoder encoder) throws CodingException{
    super.encode(encoder);
    encoder.encodeObject(CodableCircleView.CIRCLES,circles);
  }

  // Called after unarchiving is finished.
  // Here, we just call draw().
  public void finishDecoding(){ draw();}
}
